-- Code VHDL qui effectue le dcodage des deux codeurs avec un rsolution 
-- multiplie par 2 : 400 points par tour.

-- Le signal D/G est multiplex en tenant compte du signal RD

-- Machine  tat 1 : codeur gauche
-- Machine  tat 2 : codeur droit
-- Machine  tat 3 : signal RD sur front descandant de CLK


library IEEE;                           -- but may need other libraries

use IEEE.std_logic_1164.all;            -- basic logic types
USE work.STD_ARITH.all;


--------------------- Definition des ports ----------------------------

entity Codeurs is port(

	Clk:		in std_logic;		-- Horloge principale du quartz
	RD:			in std_logic;		-- Ordre de lecture des donnes
	CGsigA: 	in std_logic;		-- Codeurs gauche canal A
	CDsigA: 	in std_logic;		-- Codeurs droit canal A
	CGsigB: 	in std_logic;		-- Codeurs gauche canal A
	CDsigB: 	in std_logic;		-- Codeurs droit canal A

	HL:			in std_logic;		-- Bits de poids fort / faible
	DG:			in std_logic;		-- Commande droite/gauche

	SensD:		in std_logic;		-- Sens du moteur droit
	SensG:		in std_logic;		-- Sens du moteur droit

	Data: 		out std_logic_vector(7 downto 0) );		-- Donne retourne au micro

	------------------------------------- Attribution des pins -----------------------------------
	attribute pin_numbers of Codeurs: entity is 
	"Clk:20 RD:23 CGsigA:62 CDsigA:65 CGsigB:45 CDsigB:41 "
	&" Data(7):24 Data(6):25 Data(5):26 Data(4):27 Data(3):28 Data(2):29 Data(1):30 Data(0):31"
	&" HL:3 DG:5 SensD:56 SensG:58";
end;




architecture Codeurs_arch of Codeurs is


-- Dfinition de la machine  4 tats pour le codeur gauche avec test de A
	-- SGA1 : A=0
	-- SGA2 : Incrment/dcrment du compteur selon B
	-- SGA3 : A=1
	-- SGA4 : Incrment/dcrment du compteur selon B
type StateGauche_type is (SGA1, SGA2, SGA3, SGA4);
signal StateGaucheA: StateGauche_type;

-- Dfinition de la machine  4 tats pour le codeur gauche signal B
	-- SGB1 : B=1
	-- SGB2 : Incrment/dcrment du compteur selon A
	-- SGB3 : B=0
	-- SGB4 : Incrment/dcrment du compteur selon A
type StateGauch_type is (SGB1, SGB2, SGB3, SGB4);
signal StateGaucheB: StateGauch_type;

-- Dfinition des signaux  d'autorisation + ou - du compteur gauche
signal Gplus1,Gplus2,Gmoins1,Gmoins2: std_logic;
attribute synthesis_off of Gplus1:signal is true;
attribute synthesis_off of Gplus2:signal is true;
attribute synthesis_off of Gmoins1:signal is true;
attribute synthesis_off of Gmoins2:signal is true;
---------------------------------------------------------------------------
-- Dfinition de la machine  4 tats pour le codeur droit avec test de A
	-- SDA1 : A=0
	-- SDA2 : Incrment/dcrment du compteur selon B
	-- SDA3 : A=1
	-- SDA4 : Incrment/dcrment du compteur selon B
type StateDroitA_type is (SDA1, SDA2, SDA3, SDA4);
signal StateDroitA: StateDroitA_type;
-- Dfinition de la machine  4 tats pour le codeur droit avec test de B
	-- SDB1 : B=1
	-- SDB2 : Incrment/dcrment du compteur selon A
	-- SDB3 : B=0
	-- SDB4 : Incrment/dcrment du compteur selon A
type StateDroitB_type is (SDB1, SDB2, SDB3, SDB4);
signal StateDroitB: StateDroitB_type;
-- Dfinition des signaux  d'autorisation + ou - du compteur droite
signal Dplus1,Dplus2,Dmoins1,Dmoins2: std_logic;
-- Dfinition de la machine  3 tats pour la mmorisation des donnes
-- pour la communication avec le micro
	-- SA : RD=1
	-- SB : RD=1
	-- SC : Mmorisation des donnes des codeurs

type Sreg2_type is (SA, SB,SC);
signal Sreg2: Sreg2_type;



-- Compteur 16 bits du codeurs gauche 
signal CompteurDroite	 	: std_logic_vector(15 downto 0);

-- Compteur 16 bits du codeurs gauche 
signal CompteurGauche	 	: std_logic_vector(15 downto 0);

--verrous des  compteurs
signal MemDroite	 	: std_logic_vector(15 downto 0);
signal MemGauche	 	: std_logic_vector(15 downto 0);

	begin


				-- Envoie de la donne (Gauche, bit de poids fort
	Data<=	 	MemGauche(15 downto 8)	when (HL='1') and (DG='0') else 
				-- Envoie de la donne (Gauche, bit de poids faible
				MemGauche(7 downto 0)  when (HL='0') and (DG='0') else
				-- Envoie de la donne (Droite, bit de poids fort
				MemDroite(15 downto 8)	when (HL='1') and (DG='1') else 
				-- Envoie de la donne (Droite, bit de poids faible
				MemDroite(7 downto 0)  when (HL='0') and (DG='1') else "00000000";
	
	
------------------------ Machine  tat codeur gauche en testant A-------
StateGaucheA_machine: process (Clk)
	begin
	
		if Clk'event and Clk = '1' then		-- Sur un front montant de CLK
			case StateGaucheA is
		
				when SGA1 =>Gplus1<='0';Gmoins1<='0';
				                                           -- Etat SG1 : A=0
					if (CGsigA='1') then					-- On change d'tat si A passe  1
						StateGaucheA <= SGA2;			
					end if;
		
				when SGA2 =>					-- Etat SG2 : transition A=0 -> A=1
					if (CGsigB='1') then					-- Selon la valeur de B
						Gplus1<='1';Gmoins1<='0';	-- on incrmente le compteur
					else									-- ou 
						Gplus1<='0';Gmoins1<='1';	-- on dcrmente le compteur
					end if;
					StateGaucheA <= SGA3;							-- puis on passe dans l'tat SG3
				
				when SGA3 =>Gplus1<='0';Gmoins1<='0';			-- Etat SG3 : A=1
					if (CGsigA='0') then					-- On change d'tat si A passe  0
						StateGaucheA <= SGA4;
					end if;
		
				when SGA4 =>					-- Etat SG4 : transistion A=1 -> A=0
					if (CGsigB='0') then					-- Selon la valeur de B
						Gplus1<='1';Gmoins1<='0';	-- on incrmente le compteur
					else									-- ou
						Gplus1<='0';Gmoins1<='1';	-- on dcrmente le compteur
					end if;
					StateGaucheA <= SGA1;							-- Puis on passe dans l'tat SG1
		
				when others =>				-- En cas de problme ?
					StateGaucheA <= SGA1;			-- On retourne en SG1
		
			end case;
		end if;
end process;

	------------------------ Machine  tat codeur gauche en testant B----------
StateGaucheB_machine: process (Clk)
	begin
		if Clk'event and Clk = '1' then		-- Sur un front montant de CLK
			case StateGaucheB is
				when SGB1 =>Gplus2<='0';Gmoins2<='0';
				                                           -- Etat SG1 : A=0
					if (CGsigB='1') then					-- On change d'tat si A passe  1
						StateGaucheB <= SGB2;			
					end if;
		
				when SGB2 =>					-- Etat SG2 : transition A=0 -> A=1
					if (CGsigA='0') then					-- Selon la valeur de B
						Gplus2<='1';Gmoins2<='0';	-- on incrmente le compteur
					else									-- ou 
					Gplus2<='0';Gmoins2<='1';	-- on dcrmente le compteur
					end if;
					StateGaucheB <= SGB3;							-- puis on passe dans l'tat SG3
				
				when SGB3 =>Gplus2<='0';Gmoins2<='0';					-- Etat SG3 : A=1
					if (CGsigB='0') then					-- On change d'tat si A passe  0
						StateGaucheB <= SGB4;
					end if;
		
				when SGB4 =>					-- Etat SG4 : transistion A=1 -> A=0
					if (CGsigA='1') then					-- Selon la valeur de B
						Gplus2<='1';Gmoins2<='0';	-- on incrmente le compteur
					else									-- ou
						Gplus2<='0';Gmoins2<='1';	-- on dcrmente le compteur
					end if;
					StateGaucheB <= SGB1;							-- Puis on passe dans l'tat SG1
		
				when others =>				-- En cas de problme ?
					StateGaucheB <= SGB1;			-- On retourne en SG1
		
			end case;
		end if;
end process;
  -----------------------------COMPTEUR GAUCHE---------------------------
CompteurGauche: process (Clk)
	begin
		if Clk'event and Clk = '1' then		-- Sur un front montant de CLK
           if (Gplus1='1' or Gplus2='1') then 
		                    CompteurGauche<=CompteurGauche+1;
		    elsif (Gmoins1='1' or Gmoins2='1')then
			                CompteurGauche<=CompteurGauche-1;
		   end if;
		end if;
 end process;
 -------------------------- Machine  tat codeur droit AVEC TEST DE A------------ 
StateDroitA_machine: process (Clk)
	begin
	
		if Clk'event and Clk = '1' then		-- Sur un front montant de CLK
			case StateDroitA is
		
				when SDA1 =>Dplus1<='0';Dmoins1<='0';					-- Etat SD1 : A=0
					if (CDsigA='1') then					-- On change d'tat si A passe  1
						StateDroitA <= SDA2;			
					end if;
		
				when SDA2 =>					-- Etat SD2 : transition A=0 -> A=1
					if (CDsigB='1') then					-- Selon la valeur de B
						Dplus1<='1';Dmoins1<='0';	-- on incrmente le compteur
					else									-- ou 
						Dplus1<='0';Dmoins1<='1';	-- on dcrmente le compteur
					end if;
					StateDroitA <= SDA3;							-- puis on passe dans l'tat SD3
				
				when SDA3 =>Dplus1<='0';Dmoins1<='0';					-- Etat SD3 : A=1
					if (CDsigA='0') then					-- On change d'tat si A passe  0
						StateDroitA <= SDA4;
					end if;
		
				when SDA4 =>					-- Etat SD4 : transistion A=1 -> A=0
					if (CDsigB='0') then					-- Selon la valeur de B
						Dplus1<='1';Dmoins1<='0';	-- on incrmente le compteur
					else									-- ou 
						Dplus1<='0';Dmoins1<='1';	-- on dcrmente le compteur
					end if;
					StateDroitA <= SDA1;						-- Puis on passe dans l'tat SD1
		
				when others =>				-- En cas de problme ?
					StateDroitA <= SDA1;			-- On retourne en SD1
		
			end case;
		end if;
end process;
-------------------------- Machine  tat codeur droit AVEC TEST DE B------------ 
StateDroitB_machine: process (Clk)
	begin
	
		if Clk'event and Clk = '1' then		-- Sur un front montant de CLK
			case StateDroitB is
		
				when SDB1 =>Dplus2<='0';Dmoins2<='0';			-- Etat SD1 : B=0
					if (CDsigB='1') then					-- On change d'tat si A passe  1
						StateDroitB <= SDB2;			
					end if;
		
				when SDB2 =>					-- Etat SD2 : transition B=0 -> B=1
					if (CDsigA='0') then					-- Selon la valeur de A
						Dplus2<='1';Dmoins2<='0';	-- on incrmente le compteur
					else									-- ou 
						Dplus2<='0';Dmoins2<='1';	-- on dcrmente le compteur
					end if;
					StateDroitB <= SDB3;							-- puis on passe dans l'tat SD3
				
				when SDB3 =>Dplus2<='0';Dmoins2<='0';				-- Etat SD3 : B=1
					if (CDsigB='0') then					-- On change d'tat si B passe  0
						StateDroitB <= SDB4;
					end if;
		
				when SDB4 =>					-- Etat SD4 : transistion B=1 -> B=0
					if (CDsigA='1') then					-- Selon la valeur de A
						Dplus2<='1';Dmoins2<='0';	-- on incrmente le compteur
					else									-- ou 
						Dplus2<='0';Dmoins2<='1';	-- on dcrmente le compteur
					end if;
					StateDroitB <= SDB1;						-- Puis on passe dans l'tat SD1
		
				when others =>				-- En cas de problme ?
					StateDroitB <= SDB1;			-- On retourne en SD1
		
			end case;
		end if;
end process;
  -----------------------------COMPTEUR DROIT---------------------------
CompteurDroit: process (Clk)
	begin
		if Clk'event and Clk = '1' then		-- Sur un front montant de CLK
           if (Dplus1='1' or Dplus2='1') then 
		                    CompteurDroite<=CompteurDroite+1;
		    elsif (Dmoins1='1' or Dmoins2='1')then
			                CompteurDroite<=CompteurDroite-1;
		   end if;
		end if;
 end process;
	---------------- Machine  tat dialogue pic --------------------------
process(clk)
  begin
		if (Clk'event and Clk = '0') then		-- Sur un front descendant de CLK
			case Sreg2 is
		
				when SA =>					-- Etat SA : attente de RD=0
					if (RD='0') then		-- On change d'tat si rd passe  0
						Sreg2 <= SB;			
					end if;
		
				when SB =>					-- Etat SB : attente de RD=1
					if (RD='1') then		-- On change d'tat si rd passe  1
						Sreg2 <= SC;
					end if;
												
				when SC =>					-- Etat SC : Mmorisation
						Memgauche<=CompteurGauche;	-- on mmorise CptGauche
	
						Memdroite<=CompteurDroite;  -- on mmorise CptDroite
						Sreg2 <= SA;		-- On passe dans l'tat SA

				when others =>						-- En cas de problme ?
					Sreg2 <= SA;					-- On retourne en SA
		
			end case;
		end if;
end process;


end Codeurs_arch;
